/* copyright-start
  ******************************************************************************
  *  Licensed Material - Property of IBM
  *
  *  5724-V27, 5655-R10, 5655-M22, 5655-L21, 5655-I49, 5655-D92
  *  Copyright IBM Corp. 1993, 2009                          
  *
  *  The source code for this program is not published or otherwise
  *  divested of its trade secrets, irrespective of what has been
  *  deposited with the U.S. Copyright Office.
  ******************************************************************************
  * copyright-end */

package com.ibm.dmh.rest.client;

import java.net.*;
import java.util.*;

import org.jdom.*;

/** Define a datatype to store the required information to upload an artifact to RAM **/
public class DmhArtifact {
	
	private String restDetailsUri = null;
	DmhRestClientDetails artifactDetails = null;
	
	private DmhArtifact() {
	}

	public DmhArtifact(String detailsUri, String u, String p) {
		this.restDetailsUri = detailsUri;
		this.artifactDetails = new DmhRestClientDetails(restDetailsUri, u, p);  
	}
	
   /**
    * @return RAA assigned RAM GUID 
    */
   public String getRamGUID() {
      String ramGUID = null;
      artifactDetails.initialize(); // load from RAA if not loaded (optimization)
      String type = artifactDetails.getAttribute("type");
      String id   = artifactDetails.getAttribute("id");
      URI raaHomeUri = getRaaUri();         
      ramGUID = "raa-"+type+"-"+id+"-"+raaHomeUri.getHost()+"-"+raaHomeUri.getPort();
//      String a2 = raaHomeUri.getPath();         // /dmhrest/xml/Application/1075719494
//      String a4 = raaHomeUri.getHost();         // wsaa.svl.ibm.com
//      String ab = raaHomeUri.getScheme();       // http
      return ramGUID;
   }

   /**
	 * 
	 * @return all attributes formatted
	 */
	public String getRaaAttributesFormatted() {
      return getRaaAttributesFormatted(null);
	}

   /**
    * @param excludedAttributeNames
    * @return
    */
   public String getRaaAttributesFormatted(Set excludedAttributeNames) {
      // get all attributes formatted
      StringBuffer description = new StringBuffer();
      // Iterate over attributes
      List<Element> attribs = artifactDetails.getAttributes();
      for (Iterator aIter = attribs.iterator(); aIter.hasNext();) {
         Element attr = (Element) aIter.next();
         // The Set is a set of strings - names of attributes to exclude
         // if the exclude set is not provided or it does not contain excluded name append 
         if (( excludedAttributeNames == null) || (! excludedAttributeNames.contains(attr.getName()))) {
            description.append(attr.getName()).append("=").append(attr.getText()).append("<br/>\n");            
         }
      }     
      return description.toString();
   }

   
   
   public String getRamName() {
      artifactDetails.initialize();
//      String name = "RAA "+artifactDetails.getAttribute("type")+":"+artifactDetails.getAttribute("Name")+" on "+getRaaUri().getHost()+":"+getRaaUri().getPort();     
//      String name = artifactDetails.getAttribute("type")+": "+artifactDetails.getAttribute("Name");     
      String name = artifactDetails.getAttribute("Name");
  		return name;
	}

   private URI getRaaUri() {
      URI raaHomeUri = null;
      try {
         raaHomeUri = new URI(restDetailsUri);
      } catch (URISyntaxException e) {
         throw new RuntimeException("Malformed RAA REST details URI.", e);
      }
      return raaHomeUri;
   }
	public static String getRamAssetType() {
		String ramAssetType = "RAA Asset Type";
		return ramAssetType;
	}
	public static String getRamCommunityName() {
	    String ramCommunity = "RAA Community";
		return ramCommunity;
	}
	public static String getRamCategorySchemaName() {
		return "RAA Category Schema";
	}
	public static String getRamRaaAttributesName() {
	    String ramCommunity = "RAA Attributes";
		return ramCommunity;
	}
	public static String getRamRaaAttributesNameDetailsPage() {
		return "RAA Details Page";
	}
	public static String getRamCategoryModelName() {
		return "RAA Type";
	}
	public String getCategoryName() {
		return artifactDetails.getAttribute("type");
	}
	public String getRamOwner() {
		String ramOwner = "RAA";
		return ramOwner;
	}
	public String getRamShortDescription() {
//		String shortDescription = "RAA "+artifactDetails.getAttribute("type")+":"+artifactDetails.getAttribute("Name");
//    String name = "RAA "+artifactDetails.getAttribute("type")+":"+artifactDetails.getAttribute("Name")+" on "+getRaaUri().getHost()+":"+getRaaUri().getPort();     
      String shortDescription = "RAA "+artifactDetails.getAttribute("type")+": "+artifactDetails.getAttribute("Name")
             +" at "+restDetailsUri; 
//      +" on "+getRaaUri().getHost()+":"+getRaaUri().getPort(); 
		return shortDescription;
	}
	public static String getVersion() {
		String version = "1.0";
		return version;
	}
	public String getRaaDetailsPageLink() {
		String detailsLinkHtml = "<a href=\""+artifactDetails.getAttribute("DetailsURL")+"\">RAA Details Page</a>"; 
		return detailsLinkHtml;
	}

   /**
    * @return URI to RAA home web page
    */
   public String getRaaHomepage() {
		// get the protocol and ip address only
		int endPos = restDetailsUri.indexOf("dmhrest");
		return restDetailsUri.substring(0, endPos)+"dmh";
	}

   /**
    * @return html formatted link to RAA home web page
    */
   public String getRaaHomepageLink() {
      return "<a href=\""+getRaaHomepage()+"\">"+getRaaHomepage()+"</a>";
   }   
   
   /**
    * @param raaAttributename
    * @return
    */
   public String getAttributeValue(String raaAttributename) {
      return artifactDetails.getAttribute(raaAttributename);
   }

   /**
    * @param relationshipName
    * @return
    */
   public static String makeRamRelationshipName(String raaRelationshipName) {
      return "RAA "+raaRelationshipName;
   }

   /**
    * @param relationshipName
    * @return
    */
   public static String makeRamAttributeName(String raaAttributeName) {
      return "RAA "+raaAttributeName;
   }

   /**
    * @return
    */
   public static String getRamRaaAttributesNameRaaHome() {
      return "RAA Home";
   }

   /**
    * @param name
    * @return
    */
   public static String makeRamAttributeDescription(String name) {
      return "RAA attribute: "+name;
   }
}