/* copyright-start
  ******************************************************************************
  *  Licensed Material - Property of IBM
  *
  *  5724-V27, 5655-R10, 5655-M22, 5655-L21, 5655-I49, 5655-D92
  *  Copyright IBM Corp. 1993, 2009                          
  *
  *  The source code for this program is not published or otherwise
  *  divested of its trade secrets, irrespective of what has been
  *  deposited with the U.S. Copyright Office.
  ******************************************************************************
  * copyright-end */

package com.ibm.dmh.rest.client;

import java.util.*;

import org.jdom.*;
import org.jdom.filter.*;

public class DmhRestClientDetails extends DmhRestClient {

	private String dmhRestDetailsUri = null;
   private String user = null;
   private String pswd = null;

	private DmhRestClientDetails() {
	}

	public DmhRestClientDetails(String detailsUri, String user, String pswd) {
		dmhRestDetailsUri = detailsUri;
      this.user = user; 
      this.pswd = pswd;
	}

   /**
    * Optional method - forces network communication to obtain the resource 
    * representation.  
    *
    */
	public void initialize() {
      if (doc == null ) doc = parseDoc(dmhRestDetailsUri, user, pswd);  
   }
   
   @SuppressWarnings("unchecked")
	public List<Element> getRelationships() {
		if (doc == null ) doc = parseDoc(dmhRestDetailsUri, user, pswd);
		Filter f = new ElementFilter("relationships");
		// get element that encloses the links to relationships
		Element relationshipsElement = (Element) doc.getRootElement().getDescendants(f).next();
		List<Element> relationships = relationshipsElement.getChildren();
		return relationships;
	}

	@SuppressWarnings("unchecked")
	public List<Element> getAttributes() {
		if (doc == null ) doc = parseDoc(dmhRestDetailsUri, user, pswd);
		Filter f = new ElementFilter("attributes");
		Element attributesElement = (Element) doc.getRootElement().getDescendants(f).next();
		List<Element> attributes = attributesElement.getChildren();
		return attributes;
	}

	/**
	 * Returns attribute value 
	 * @param attributeName
	 * @return
	 */
	public String getAttribute(String attributeName) {		
		String value = "";
		// Optimize - if doc == null - retrieve only the single attribute resource; leave doc as null 
		if (doc == null ) { // ? details representation already obtained
			value = DmhRestClientDetails.getAttribute(attributeName, dmhRestDetailsUri, user, pswd);
		} else {            // details representation already in memory 
			Filter f = new ElementFilter(attributeName);
			Element attributesElement = (Element) doc.getRootElement().getDescendants(f).next();
			value = attributesElement.getText();
		}
		return value;
	}

	public static String getAttribute(String attributeName, String restDetailsUri, String userId, String password) {
		String value = "";
		String attributeUri = restDetailsUri;
		if (! attributeUri.endsWith("/") ) attributeUri = attributeUri + "/"; 
		attributeUri = attributeUri+"attribute/"+attributeName;
		Document d = createDocument(attributeUri, userId, password);
		Filter f = new ElementFilter(attributeName);
		Element attributesElement = (Element) d.getRootElement().getDescendants(f).next();
		value = attributesElement.getText();
		return value;
	}

	/**
	  * Tests for DmhRestClientDetails can be found at com.ibm.ram.test DmhRestClientDetailsTest.java
	  */
	public static void main(String[] args) {
//		DmhRestClientDetails c = new DmhRestClientDetails("http://wsaa.svl.ibm.com:9080/dmhrest/xml/Program/893390962");
//		// Iterate over attributes
//		List<Element> a = c.getAttributes();
//		for (Iterator iter = a.iterator(); iter.hasNext();) {
//			Element attr = (Element) iter.next();			
//			System.out.println("Attribute "+attr.getAttributeValue("rel")+"="+attr.getText());
//		}
//		// Iterate over relationships
//		List<Element> r = c.getRelationships();
//		for (Iterator iter = r.iterator(); iter.hasNext();) {
//			Element rel = (Element) iter.next();			
//			System.out.println("Relationship "+rel.getText()+"="+rel.getAttributeValue("href"));
//		}
	}


}
