/* copyright-start
  ******************************************************************************
  *  Licensed Material - Property of IBM
  *
  *  5724-V27, 5655-R10, 5655-M22, 5655-L21, 5655-I49, 5655-D92
  *  Copyright IBM Corp. 1993, 2009                          
  *
  *  The source code for this program is not published or otherwise
  *  divested of its trade secrets, irrespective of what has been
  *  deposited with the U.S. Copyright Office.
  ******************************************************************************
  * copyright-end */

package com.ibm.dmh.rest.client;

import java.util.*;

import org.jdom.*;
import org.jdom.filter.*;

/**
 * Given a uri to RAA xml REST service allow to retrieve paged list 
 * of RAA artifacts of a given type
 */
public class DmhRestClientPagedListOfLinks extends DmhRestClient{
	private int pagesRead = 0;
	private String dmhPagedListOfLinksUri = "";
	private String nextPageUri = null;

   private String raaUser = null;
   private String raaPswd = null;
   
	/**
     * Given a uri to RAA xml REST service paged summary of RAA artifacts of a given type
	 * @param dmhRestRootUri
	 * @param type
	 */
	public DmhRestClientPagedListOfLinks(String pagedListUri, String user, String password) {
		setDmhPagedListOfLinksUri(pagedListUri);
      raaUser = user; 
      raaPswd = password;
		pagesRead =0; 
	}

	/**
	 * No default constructor - must be instantiated with uri  
	 */	
	private DmhRestClientPagedListOfLinks() {
	}

//	/**
//	 * Reads elements from the current page - advances to the next page 
//	 * @return
//	 */ 
//	@SuppressWarnings("unchecked")
//	public List<Element> getPageArtifactElements() {
//		List<Element> assets = new ArrayList();	
//		// from the xml doc retrive all included links to artifact details
//      if( (doc != null) && (doc.getRootElement() != null)) {
//         Filter f = new ElementFilter("artifact");
//         Iterator<Element> artifactIt = doc.getRootElement().getDescendants(f);
//         while (artifactIt.hasNext()) {
//            assets.add(artifactIt.next());			
//         }      
//      }
//	
//		pagesRead ++;
//		return assets;		
//	}

   /**
    * Reads elements from the current page - advances to the next page 
    * @return
    */ 
   @SuppressWarnings("unchecked")
   public List<Element> getPageArtifactElements() {
      List<Element> assets = new ArrayList();   
      // from the xml doc retrive all included links to artifact details
      if( (doc != null) && (doc.getRootElement() != null)) {
         Filter f = new ElementFilter("page");
         Iterator<Element> artifactIt = doc.getRootElement().getDescendants(f);
         if (artifactIt.hasNext()) {
            Element pageElement = artifactIt.next();         
            f = new ElementFilter("artifact");
            artifactIt = pageElement.getDescendants(f);
            while (artifactIt.hasNext()) {
               assets.add(artifactIt.next());         
            }      
         }         
      }
   
      pagesRead ++;
      return assets;    
   }

	/**
	 * Loads next page 
	 * @return
	 */ 
	@SuppressWarnings("unchecked")
	public boolean nextPage() {
		if (nextPageUri == null) 
			return false;

		doc = parseDoc(nextPageUri, raaUser, raaPswd);
		nextPageUri = getNextPageUri(); 		// get next page link 
		
		return true; 		
	}
	
	public List<String> getPageLinks() {
		List<String> uris = new ArrayList<String>(25);
		
		List<Element> linksPage = getPageArtifactElements();
		// For each page of atrifatc links - create artifact (if not already exist) 
		// and the realtionship  
		for (Iterator iPage = linksPage.iterator(); iPage.hasNext();) {
			Element e = (Element) iPage.next();
			String relatedRestLink = e.getAttributeValue("href");
			uris.add(relatedRestLink);
		}			
		return uris;
	}
	

	
	
	private String getNextPageUri() {
		String nextPageLink = null;
		try {
			Filter f = new ElementFilter("next");
			if (doc.getRootElement().getDescendants(f).hasNext()) {
				Element e = (Element) doc.getRootElement().getDescendants(f).next();
				nextPageLink = e.getAttributeValue("href");				
			}	
		} catch (Exception e) {
			nextPageLink = null; // no next page		
		}
		return nextPageLink;
	}

	public int getPageNo() {
		return pagesRead;
	}

	public String getDmhPagedListOfLinksUri() {

		return dmhPagedListOfLinksUri;
	}

	public void setDmhPagedListOfLinksUri(String uri) {
		this.dmhPagedListOfLinksUri = uri;
		this.nextPageUri = uri;
	}
	
	public boolean hasNextPage() {
		if (nextPageUri == null) 
			return false;
		else
			return true;
	}

	public static void main(String[] args) {
		DmhRestClientPagedListOfLinks c = new DmhRestClientPagedListOfLinks("http://wsaa.svl.ibm.com:9080/dmhrest/xml", "u", "p");
		// Iterate over all pages
		while (c.nextPage()) {
			List<Element> pgms = c.getPageArtifactElements();
			for (Iterator iter = pgms.iterator(); iter.hasNext();) {
				Element e = (Element) iter.next();
				System.out.print(e.getText()+"="+e.getAttributeValue("href")+"\n");								
			}			
		}
	}

}
